/*
 * Copyright 2010 Florian Fischer <florianfischer@gmx.de>
 *
 * This file is part of NetSurf, http://www.netsurf-browser.org/
 *
 * NetSurf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * NetSurf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NETSURF_FRAMEBUFFER_BOOKMARKS_H_
#define _NETSURF_FRAMEBUFFER_BOOKMARKS_H_

struct fb_bookmark_group_s;

struct fb_bookmark_s {
        struct fb_bookmark_group_s *group;
        char *url;
        char *title;
        
        struct fb_bookmark_s *next;
};

struct fb_bookmark_group_s {
        char *name;
        struct fb_bookmark_s *entry; 
        
        struct fb_bookmark_group_s *next;
}; 


typedef struct fb_bookmark_s fb_bookmark_t;
typedef struct fb_bookmark_group_s fb_bookmark_group_t;

extern void bookmarks_read(const char *file);

/** Access to the bookmarks. Use for reading and display only! */
extern fb_bookmark_group_t *bookmarks_get(void);
/** Add a new bookmark group. */
extern fb_bookmark_group_t *bookmarks_add_group(const char *name);
/** Add a new bookmark to the given group. */
extern fb_bookmark_t *bookmarks_add(fb_bookmark_group_t *group, const char *url, const char *title);
/** Delete a bookmark. */
extern void bookmarks_delete(fb_bookmark_t *entry);
/** Update a bookmark with new information. */
extern void bookmarks_update(fb_bookmark_t *entry, const char *url, const char *title);

/** Have the bookmarks been modified thru the above functions? */
extern bool bookmarks_changed(void); 
/** Saves the bookmarks to the file from which they were loaded. 
  * Does nothing if the bookmarks have not been changed. */
extern void bookmarks_save(void); 
/** Frees the loaded bookmarks. */
extern void bookmarks_free(void);


#endif 
