/*
 * Copyright 2008 Vincent Sanders <vince@simtec.co.uk>
 *
 * This file is part of NetSurf, http://www.netsurf-browser.org/
 *
 * NetSurf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * NetSurf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NETSURF_FB_GUI_H
#define NETSURF_FB_GUI_H

/* VKB begin */
typedef enum vkb_mode_e {
        FB_VKB_123,
        FB_VKB_abc,
        FB_VKB_Abc,
        FB_VKB_ABC
} vkb_mode_t;
#define FB_VKB_MODE_COUNT 4

// number of accepted input keys
#define FB_VKB_NUM_KEYS 10
// max. number of items per key
#define FB_VKB_NUM_ITEMS 10 

typedef struct vkb_state_s {
        vkb_mode_t mode;
        int current_key; // number of last pressed key (0-9) or -1 if none
        int current_item; // item in list of current key
        int output[FB_VKB_NUM_KEYS][FB_VKB_NUM_ITEMS]; // configured output Unicode characters for each key
        int timeout_ms; // timeout for key input 
} vkb_state_t; 

/* VKB end */




typedef struct fb_cursor_s fb_cursor_t;

/* bounding box */
typedef struct bbox_s {
        int x0;
        int y0;
        int x1;
        int y1;
} bbox_t;

typedef struct framebuffer_s {
        int width;
        int height;
        uint8_t *ptr; /**< Base of video memory. */
        int linelen; /**< length of a video line. */
        int bpp;
        colour palette[256]; /* palette for index modes */
        fb_cursor_t *cursor;
} framebuffer_t;

struct gui_window {
        struct browser_window *bw;

        struct fbtk_widget_s *window;
        struct fbtk_widget_s *menu;                /**< added element which folds out a menu */
        struct fbtk_widget_s *back;
        struct fbtk_widget_s *forward;
        struct fbtk_widget_s *url;
        struct fbtk_widget_s *status;
        struct fbtk_widget_s *vkb;
        struct fbtk_widget_s *navmode; /**< added element which changes nav mode */
        struct fbtk_widget_s *throbber;
        struct fbtk_widget_s *hscroll;
        struct fbtk_widget_s *vscroll;
        struct fbtk_widget_s *browser;
        int throbber_index;
        // caret support
        int c_x;
        int c_y;
        int c_h;
        // vkb support
        vkb_state_t vkb_state;
};

/* VKB STUFF **************** */
/* is the user currently writing some text? */
bool fb_is_writing_text(struct gui_window* gw); 
/** Loads configuration */
void fb_vkb_init(struct gui_window* gw); 
/** Toggle VKB mode */
void fb_toggle_vkb_mode(struct gui_window* gw);
/** Should be called when an ASCII key (not a function key!) is pressed.
   @param ch The unicode character that was entered
   @return The unicode character that is input just now. May return 0, which means 'you need to wait first'
*/
int fb_vkb_input(struct gui_window* gw, int ch);
/** Should be called after the timeout elapsed 
  * @return The unicode character to input now. May return 0, which means no character is pending. 
*/
int fb_vkb_timeout(struct gui_window* gw); 
/** Get currently pending character, or 0 if none */
int fb_vkb_get_current(struct gui_window* gw);

/* VKB END *************** */

extern framebuffer_t *framebuffer;
extern struct gui_window *window_list;

/* (keyboard) navigation */
enum fb_nav_mode {
        FB_NAV_LINK,
        FB_NAV_SCROLL_PAGE,
        FB_NAV_SCROLL,
        FB_NAV_MOVE_POINTER
}; 
#define FB_MOVE_POINTER_DIST 16
enum fb_nav_dir {
        FB_NAV_UP,
        FB_NAV_DOWN,
        FB_NAV_LEFT,
        FB_NAV_RIGHT
};
extern enum fb_nav_mode fb_nav_mode; 


/* scroll a window */
void fb_window_scroll(struct gui_window *g, int x, int y);
bool gui_window_get_scroll_with_pan(struct gui_window *g, int *sx, int *sy);

#endif /* NETSURF_FB_GUI_H */

/*
 * Local Variables:
 * c-basic-offset:8
 * End:
 */


