/*
 * Copyright 2008 Daniel Silverstone <dsilvers@netsurf-browser.org>
 *
 * This file is part of NetSurf, http://www.netsurf-browser.org/
 *
 * NetSurf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * NetSurf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _NETSURF_FRAMEBUFFER_OPTIONS_H_
#define _NETSURF_FRAMEBUFFER_OPTIONS_H_

#include "desktop/options.h"

extern int option_fb_depth;
extern int option_fb_refresh;
extern bool option_fb_font_monochrome; /* render font monochrome */
extern char *option_fb_device;
extern char *option_fb_input_devpath;
extern char *option_fb_input_glob;
extern char *option_fb_face_sans_serif; /* default sans face */
extern char *option_fb_face_sans_serif_bold; /* bold sans face */
extern char *option_fb_face_sans_serif_italic; /* bold sans face */
extern char *option_fb_face_sans_serif_italic_bold; /* bold sans face */
extern char *option_fb_face_monospace; /* monospace face */
extern char *option_fb_face_serif; /* serif face */
extern char *option_fb_face_serif_bold; /* bold serif face */
extern int option_fb_mouse_delay; /* addition: mouse delay */
extern bool option_fb_key_repeat; /* addition: enable key repetition? */
extern int option_fb_key_repeat_delay;  /* delay before starting key repeat */
extern int option_fb_key_repeat_interval; /* interval between repeated keys */
extern char *option_fb_search_url; /* addition: search URL */ 
extern bool option_fb_vkb_enabled; /* addition: enable VKB? */
extern int option_fb_vkb_timeout_ms; /* VKB timeout msec */
extern char *option_fb_vkb[10]; /* VKB char mapping */
extern char *option_fb_help_url; /* help URL */
extern bool option_fb_log_enabled; /* enable logging? */

#define FB_DEFAULT_SEARCH_URL "http://google.com/search?q="
#define FB_HELP_URL "http://sourceforge.net/apps/mediawiki/samygo/?title=NetSurf_Web_Browser"

#define EXTRA_OPTION_DEFINE                     \
  int option_fb_depth = 32;                     \
  int option_fb_refresh = 70;                   \
  bool option_fb_font_monochrome = false;       \
  char *option_fb_device = 0;                   \
  char *option_fb_input_devpath = 0;            \
  char *option_fb_input_glob = 0;               \
  char *option_fb_face_sans_serif;              \
  char *option_fb_face_sans_serif_bold;         \
  char *option_fb_face_sans_serif_italic;       \
  char *option_fb_face_sans_serif_italic_bold;  \
  char *option_fb_face_monospace;               \
  char *option_fb_face_serif;                   \
  char *option_fb_face_serif_bold;              \
  int option_fb_mouse_delay;                    \
  bool option_fb_key_repeat = true;             \
  int option_fb_key_repeat_delay = 500;         \
  int option_fb_key_repeat_interval = 30;       \
  char *option_fb_search_url = 0;               \
  bool option_fb_vkb_enabled = true;            \
  int option_fb_vkb_timeout_ms = 1500;          \
  char *option_fb_vkb[10] = {0};                \
  char *option_fb_help_url = 0;                 \
  bool option_fb_log_enabled = true;
  

#define EXTRA_OPTION_TABLE                                              \
    { "fb_depth", OPTION_INTEGER, &option_fb_depth },                   \
    { "fb_refresh", OPTION_INTEGER, &option_fb_refresh },               \
    { "fb_device", OPTION_STRING, &option_fb_device },                  \
    { "fb_input_devpath", OPTION_STRING, &option_fb_input_devpath },    \
    { "fb_input_glob", OPTION_STRING, &option_fb_input_glob },          \
    { "fb_font_monochrome", OPTION_BOOL, &option_fb_font_monochrome },  \
    { "fb_face_sans_serif", OPTION_STRING, &option_fb_face_sans_serif }, \
    { "fb_face_sans_serif_bold", OPTION_STRING, &option_fb_face_sans_serif_bold }, \
    { "fb_face_sans_serif_italic", OPTION_STRING, &option_fb_face_sans_serif_italic }, \
    { "fb_face_sans_serif_italic_bold", OPTION_STRING, &option_fb_face_sans_serif_italic_bold }, \
    { "fb_face_monospace", OPTION_STRING, &option_fb_face_monospace },  \
    { "fb_face_serif", OPTION_STRING, &option_fb_face_serif },          \
    { "fb_serif_bold", OPTION_STRING, &option_fb_face_serif_bold },     \
    { "fb_key_repeat", OPTION_BOOL, &option_fb_key_repeat },            \
    { "fb_key_repeat_delay", OPTION_INTEGER, &option_fb_key_repeat_delay }, \
    { "fb_key_repeat_interval", OPTION_INTEGER, &option_fb_key_repeat_interval }, \
    { "fb_mouse_delay", OPTION_INTEGER, &option_fb_mouse_delay },       \
    { "fb_search_url", OPTION_STRING, &option_fb_search_url },          \
    { "fb_vkb_enabled", OPTION_BOOL, &option_fb_vkb_enabled },       \
    { "fb_vkb_timeout_ms", OPTION_INTEGER, &option_fb_vkb_timeout_ms }, \
    { "fb_vkb_0", OPTION_STRING, &option_fb_vkb[0] },                   \
    { "fb_vkb_1", OPTION_STRING, &option_fb_vkb[1] },                   \
    { "fb_vkb_2", OPTION_STRING, &option_fb_vkb[2] },                   \
    { "fb_vkb_3", OPTION_STRING, &option_fb_vkb[3] },                   \
    { "fb_vkb_4", OPTION_STRING, &option_fb_vkb[4] },                   \
    { "fb_vkb_5", OPTION_STRING, &option_fb_vkb[5] },                   \
    { "fb_vkb_6", OPTION_STRING, &option_fb_vkb[6] },                   \
    { "fb_vkb_7", OPTION_STRING, &option_fb_vkb[7] },                   \
    { "fb_vkb_8", OPTION_STRING, &option_fb_vkb[8] },                   \
    { "fb_vkb_9", OPTION_STRING, &option_fb_vkb[9] },                   \
    { "fb_help_url", OPTION_STRING, &option_fb_help_url },              \
    { "fb_log_enabled", OPTION_BOOL, &option_fb_log_enabled }
    
#endif
